clear all;
more off;


%%%%%%%%%%%%%%%% MATRIX 1 %%%%%%%%%%%%%%%%%%%%% 
% random Gaussian, rapidly decaying singular values (nonlinear) %
fprintf('making matrix 1..\n');
m = 2000;
n = 4000;
bin_file = '../data/A_mat1.bin'
mat_file = '../data/A_mat1.mat'
 
p = min(m,n);
if m >= n
   [U, temp] = qr(randn(m,n),0);
   [V, temp] = qr(randn(n));
else
   [U, temp] = qr(randn(m));
   [V, temp] = qr(randn(n,m),0);
end
if p>1
    S = logspace(1,-4,p);
else
    S = [1];
end
S = diag(S);
M = U*S*V';

% write to files
fprintf('write matrix M\n');
num_nnz = m*n;
fp = fopen(bin_file,'w');
fwrite(fp,m,'int32');
fwrite(fp,n,'int32');
for i=1:m
    fprintf('writing row %d of %d\n', i,m);
    for j=1:n
        fwrite(fp,M(i,j),'double');
    end
end
fclose(fp);
fprintf('writing to mat file too..\n');
save(mat_file,'M');



%%%%%%%%%%%%%%%% MATRIX 2 %%%%%%%%%%%%%%%%%%%%% 
% random Gaussian, slowly decaying singular values (linear) %
fprintf('making matrix 2..\n');
m = 2000;
n = 4000;
bin_file = '../data/A_mat2.bin'
mat_file = '../data/A_mat2.mat'
 
p = min(m,n);
if m >= n
   [U, temp] = qr(randn(m,n),0);
   [V, temp] = qr(randn(n));
else
   [U, temp] = qr(randn(m));
   [V, temp] = qr(randn(n,m),0);
end
if p>1
    S = linspace(1,0.01,p);
else
    S = [1];
end
S = diag(S);
M = U*S*V';

% write to files
fprintf('write matrix M\n');
num_nnz = m*n;
fp = fopen(bin_file,'w');
fwrite(fp,m,'int32');
fwrite(fp,n,'int32');
for i=1:m
    fprintf('writing row %d of %d\n', i,m);
    for j=1:n
        fwrite(fp,M(i,j),'double');
    end
end
fclose(fp);
fprintf('writing to mat file too..\n');
save(mat_file,'M');



%%%%%%%%%%%%%%%% MATRIX 3 %%%%%%%%%%%%%%%%%%%%% 
% sparse matrix from Sorensen & Embree paper designed for QR 
% to perform well 
fprintf('making matrix 3..\n');
m = 2000;
n = 4000;
bin_file = '../data/A_mat3.bin'
mat_file = '../data/A_mat3.mat'
 
M = sparse(m,n);
for j = 1:10
    xx = sprand(m,1,0.01);
    yy = sprand(n,1,0.01);
    M  = M + (2/j)*xx * yy';
    %    M  = M + (2/j^2)*xx * yy'; 
end
for j = 11:n
    xx = sprand(m,1,0.01);
    yy = sprand(n,1,0.01);
    M = M + (1/j)*xx * yy';
    %    M = M + (1/j^2)*xx * yy'; 
end
M = full(M);

% write to files
fprintf('write matrix M\n');
num_nnz = m*n;
fp = fopen(bin_file,'w');
fwrite(fp,m,'int32');
fwrite(fp,n,'int32');
for i=1:m
    fprintf('writing row %d of %d\n', i,m);
    for j=1:n
        fwrite(fp,M(i,j),'double');
    end
end
fclose(fp);
fprintf('writing to mat file too..\n');
save(mat_file,'M');



%%%%%%%%%%%%%%%% MATRIX 4 %%%%%%%%%%%%%%%%%%%%% 
% Kahan counterexample matrix designed for pivoted QR 
% to perform poorly 
fprintf('making matrix 4..\n');
m = 2000;
n = 2000;
bin_file = '../data/A_mat4.bin'
mat_file = '../data/A_mat4.mat'


theta=rand;
theta=pi*theta/2;

zeta=cos(theta);
phi=sin(theta);

Snvec=zeta.^(0:(n-1));
Sn=sparse(1:n,1:n,Snvec,n,n,n);

ind=logical(triu(ones(n),1)==1);
Kn=speye(n);
Kn(ind)=-phi;

M=full(Sn*Kn);


% write to files
fprintf('write matrix M\n');
num_nnz = m*n;
fp = fopen(bin_file,'w');
fwrite(fp,m,'int32');
fwrite(fp,n,'int32');
for i=1:m
    fprintf('writing row %d of %d\n', i,m);
    for j=1:n
        fwrite(fp,M(i,j),'double');
    end
end
fclose(fp);
fprintf('writing to mat file too..\n');
save(mat_file,'M');


